%eyamcovid
% The Infectives vs time curve for COVID-19 is estimated from the
% cumulative UK death record due to CV-19 between 5/3/2020 to 15/12/2021.
%
% The alpha (a) parameter is fixed, as this is used to find I from the
% cumulative deaths. The model is: d(D+R)/dt = a*I. Assume a fixed
% mortality fraction k such that D = k*(R+D).
%
% LAST UPDATED by Andy French 16/12/2021

function eyamcovid

%Characteristic time constant /days and hence alpha parameter (in days^-1),
%from Wuhan China data curve fit
T = (365/12)/3.263;
a = 1/T;

%Mortality fraction k  i.e. D = k*( D + R )
k = 0.01;

%Time window for rolling average of I(t) from the
%numerical derivative of the cumulative D vs t.
W = 7;

%Fontsize for graphs
fsize = 18;

%

%Get cumulative dead (D) and recovered (R) (in millions) from data.
%Time t in days.
[t,D,R,start_date,end_date] = get_D_R_from_data(k);

%Estimate Infective population (in millions) vs time (days)
[I,tmax,Imax] = estimate_I( t, D, k, a, W );

%Estimate basic reproduction number R0 vs time using R0 = dI/I + 1
R0 = estimate_R0(t,I);

%

%Plot t vs I
plot(t,I,'r-+'); grid on;
set(gca, 'fontsize',fsize );
xlabel(['Time in days since ',start_date]);
ylabel('Estimated UK infectives /millions');
title(['Estimated UK COVID-19 infectives ',start_date,' - ',end_date]);
print(gcf, 'UK COVID I vs t.png','-dpng','-r300' ); clf;

%Plot D vs t
plot(t,D*1000,'k+-');
grid on; set(gca, 'fontsize',fsize );
xlabel(['Time in days since ',start_date]);
ylabel('Cumulative deaths /thousands');
title(['Cumulative UK CV-19 deaths /thousands ',start_date,' - ',end_date]);
print(gcf, 'UK D vs t.png','-dpng','-r300' ); clf;

%Plot R0 vs time
plot(t,R0,'b-'); grid on; set(gca, 'fontsize',fsize ); 
xlabel(['Time in days since ',start_date]); ylabel('fractional I gain per day');
title('R_0 = \deltaI/I + 1 vs time')
print(gcf, 'UK COVID R0.png','-dpng','-r300' ); close(gcf);

%%

%Estimate basic reproduction number R0 vs time
function R0 = estimate_R0(t,I)

% Estimate R0, as defined as the 1 + fractional growth per day. Note there are
% lots of possible definitions of R0 !!
R0 = NaN( 1,length(t) );
for n=2:length(t)
    R0(n) = ( I(n) - I(n-1) )/I(n-1) + 1;
end

%%

%Get D,R (in millions) from data. Time t in days.
function [t,D,R,start_date,end_date] = get_D_R_from_data(k)

%Import UK cumulative deaths vs t (days) from Oxford World-in-data platform
if exist('CV-data.mat')==2
    load 'CV-data'
else
    %Get t, D data from Excel sheet
    [num,txt,raw] = xlsread('UK Covid-19.xlsx');
    t = num(:,1).'; D = num(:,2).';
    
    %Determine start and end dates
    start_date = raw{2,1};
    s = size(raw);
    end_date = raw{s(1),1};
    
    %Convert to millions
    D = D/1e6;
    
    %Estimate recovered (R) population
    R = D*(1-k)/k;
    
    %Save a .mat file for fast loading next time. Don't forget to delete if you
    %change the parameters such as mortality fraction k.
    save 'CV-data' 't' 'R' 'k' 'D' 'start_date' 'end_date'
end

%%

%Estimate Infective population vs time
function [I,tmax,Imax] = estimate_I( t, D, k, a, W )

%Determine rate of change of cumulative deaths, smoothed over W days. From
%this, estimate the total number of infectives /millions
I = maverage( dx_by_dt( D,t ), W )/ (k*a);

%Determine Imax and tmax (at infection peak)
Imax= max(I); i = find( I==Imax); i = i(1); tmax = t(i);

%%

%Numerical derivative
function xdot = dx_by_dt(x,t)
xdot = NaN( 1,numel(t) );
for n=2:numel(t)-1
    xdot(n) = ( x(n+1) - x(n-1) )/( t(n+1) - t(n-1) );
end

%%

%Fixed window moving average
function xav = maverage(x,W)
xav = NaN(1,numel(x));
for n=W:numel(x)
    xav(n) = mean( x( n+1-W : n ) );
end

%End of code