%bolt_100m_stats
% Calculates velocity and acceleration graphs, using numeric (cubic spline)
% differentiation, from (displacement,time) measurements for Usain Bolt's
% 2008 and 2009 record-breaking 100m runs.
%
% LAST UPDATED by Andy French Dec 2021

function bolt_100m_stats
%Plot combined (1) or single images (0)
bolt_plot(0); bolt_plot(1);

%%

function bolt_plot(plot_combined)

%Fontsize
if plot_combined == 1
    fsize = 14;
else
    fsize = 16;
end

%

%Time, distance data for Olympic and World Championships finals
%http://rcuksportscience.wikispaces.com/file/view/Analysing+men+100m+Nspire.pdf
t_2008 = [ 0.00, 1.83, 2.87, 3.78, 4.65, 5.50, 6.32, 7.14, 7.96, 8.79, 9.69 ];
t_2009 = [ 0.00, 1.89, 2.88, 3.78, 4.64, 5.47, 6.29, 7.10, 7.92, 8.75, 9.58 ];
x      = [ 0,    10,   20,   30,   40,   50,   60,   70,   80,   90,   100  ];

%Interpolate data using a cubic spline
N = 100; xx = linspace(0,100,N);
tt_2008 = interp1( x, t_2008, xx,'spline' );
tt_2009 = interp1( x, t_2009, xx,'spline' );

%Differentiate the data to get the velocity and accelerations
[v_pp,v_2008]=calculus(t_2008,x,t_2008,'diff','spline',0);
v_2008 = [0,v_2008(2:end)];
v_2008 = interp1( t_2008, v_2008, tt_2008,'spline' );
[v_pp,v_2009]=calculus(t_2009,x,t_2009,'diff','spline',0);
v_2009 = [0,v_2009(2:end)];
v_2009 = interp1( t_2009, v_2009, tt_2009,'spline' );
[a_pp,a_2008]=calculus(tt_2008,v_2008,tt_2008,'diff','spline',0);
[a_pp,a_2009]=calculus(tt_2008,v_2009,tt_2009,'diff','spline',0);

%Plot displacement, time graph with spline interpolations underlaid to
%create a smooth curve
if plot_combined==0; figure('color',[1 1 1],'name','Usain Bolt 100m stats: (t,x) graph'); end
if plot_combined==1; figure('color',[1 1 1],'name','Usain Bolt 100m stats'); end
if plot_combined==1; subplot(3,1,1); end
plot(tt_2008,xx,'k','linewidth',1);
hold on;
plot(tt_2009,xx,'r','linewidth',1);
plot(t_2008,x,'k+','markersize',fsize);
plot(t_2009,x,'r+','markersize',fsize);
set(gca,'fontsize',fsize); grid on
xlabel('time /s'); ylabel('Displacement x /m')
title('Usain Bolt 100m stats: (t,x) graph')
legend({'2008','2009'},'location','southeast','fontsize',fsize)
if plot_combined==0; print(gcf,'bolt t-x','-dpng','-r300'); close(gcf); end

%Plot velocity, time graph with spline interpolations underlaid to
%create a smooth curve
if plot_combined==0; figure('color',[1 1 1],'name','Usain Bolt 100m stats: (t,v) graph'); end
if plot_combined==1; subplot(3,1,2); end
plot(tt_2008,v_2008,'k','linewidth',1);
hold on;
plot(tt_2009,v_2009,'r','linewidth',1);
set(gca,'fontsize',fsize); grid on
xlabel('time /s'); ylabel('Velocity v /ms^{-1}');
title('Usain Bolt 100m stats: (t,v) graph');
if plot_combined==0; 
    legend({'2008','2009'},'location','southeast','fontsize',fsize)
    print(gcf,'bolt t-v','-dpng','-r300'); 
    close(gcf); 
end

%Plot acceleration, time graph with spline interpolations underlaid to
%create a smooth curve
if plot_combined==0; figure('color',[1 1 1],'name','Usain Bolt 100m stats: (t,a) graph'); end
if plot_combined==1; subplot(3,1,3); end
plot(tt_2008,a_2008,'k','linewidth',1); hold on;
plot(tt_2009,a_2009,'r','linewidth',1);
set(gca,'fontsize',fsize); grid on
xlabel('time /s'); ylabel('Acceleration a /ms^{-2}')
title('Usain Bolt 100m stats: (t,a) graph');
if plot_combined==0
    print(gcf,'bolt t-a','-dpng','-r300');
    legend({'2008','2009'},'location','northeast','fontsize',fsize);
    close(gcf); 
end
if plot_combined==1; print(gcf,'bolt 100m','-dpng','-r300'); close(gcf); end

%%

%CALCULUS
% Function that fits a piecwise polynomial to an array of [X,Y] data and
% thus computes the derivative and cumulative integral of Y(X) at all
% points X. The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.
%
% Syntax: [Y_pp,Y_calc]=calculus(X,Y,X0,type,poly,C)
%
% X        - Vector of data X variable
% Y        - Vector of data Y(X) variable
% X0       - Vector of X variables corresponding to that of the output.
% type     - 'integ' or 'diff'. Choses integration or diffrentiation of Y(X).
% poly     - 'spline' or 'hermite' . Choses type of piecewise polynomial
%            computed. Hermite polynomials tend to be better for
%            non-smooth data and tends to be 'shape preserving.'
% C        - If type=='integ', this paramater gives the initial value of the
%            integral corresponding to X(1).
%
% Y_pp     - Piecewise polynomial fitted to Y(X) evaluated at X0.
% Y_calc   - Vector of the same dimensions of X0, giving the
%            differential, or integral of Y(X) evaluated at all values of X0.
%
% Example: X=linspace(-2,2,20); Y=X.^2;
% [Y_pp,Y_int]=calculus(X,Y,X,'integ','spline',0);
% [Y_int,YY]=calculus(X,Y_int,X,'diff','spline',0);
% plot(X,Y); hold on;
% plot(X,YY,'g');
% plot(X,Y_pp,'ro');
% legend('Y=X^2','Integral then derivative of Y=X^2','PP fit of Y=X^2');
% xlabel('X'); ylabel('Y');

function [Y_pp,Y_calc]=calculus(X,Y,X0,type,poly,C)

%Compute piecewise polynomial. Cubic spline is the default
if strcmp(poly,'hermite')==1
    pp=pchip(X,Y);
else
    pp=spline(X,Y);
end

%Check if integration or differentiation is required. Differentiation is
%the default option.
if strcmp(type,'integ')==1
    
    %Check that C is a scalar
    if prod(size(C))~=1
        error('C must be a scalar!')
    end
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for integration
    sf=nco:-1:1;
    
    %Integral coefficients
    ico=[co./sf(ones(npy,1),:) zeros(npy,1)];
    
    %Integral spline has higher order
    nco=nco+1;
    
    %Integration constant
    ico(1,nco)=C;
    
    %Find constant terms in polynomials
    for k=2:npy
        ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
    end
    
    %Build pp form for integral
    ppi=mkpp(br,ico);
    
    %Evaluate integral at values of X0
    Y_calc=ppval(ppi,X0);
    
    %Evaluate piecewise polynomial
    Y_pp=ppval(pp,X0);
else
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for differentiation
    sf=nco-1:-1:1;
    
    %Derivative coefficients
    dco=sf(ones(npy,1),:).*co(:,1:nco-1);
    
    %Build pp form for derivative
    ppd=mkpp(br,dco);
    
    %Evaluate derivative at values of X0
    Y_calc=ppval(ppd,X0);
    
    %Evaluate piecewise polynomial
    Y_pp=ppval(pp,X0);
end

%End of code