%Test of a rectangular numerical integration rule
% LAST UPDATED by Andy French Dec 2021.

function rectangle1_test

%Number of strips
N = 6;

%Generate test x,y data
[x,y,Iexact] = xygen(3,7,N+1);
[xx,yy] = xygen(0,10,200);

%Perform numeric integration
[I,fig] = rectangle1(x,y);

%Overlay original curve
plot(xx,yy);

%Save graph as .png file
print( gcf, ['rectangle1 ',num2str(N),'.png'],'-dpng','-r300'); close(gcf);

%Percentage error of integral
disp(' ')
disp(' RECTANGLE1 RULE ')
disp([' Fractional error of integral = ',num2str( 100*(I-Iexact)/Iexact ),'%'] );
disp(' ')

%%

%rectangle1
% Numeric integration via a rectangular strip method.
%
% Syntax: [I,fig] = rectangle1(x,y)
%
% x,y vectors of plot data. Assume that the x values are increasing
% and they are equally spaced. The number of y elements must equal the
% number of x elements.
%
% I is the estimate of the integral between x(1) and x(end)
% fig is a handle of the plot figure which is generated.

function [I,fig] = rectangle1(x,y)

%Fontsize
fsize = 18;

%Determine strip width h. (Assume constant!)
N = numel(x) - 1;
h = ( x(end) - x(1) )/N;

%Compute estimate of integral
I = h*sum( y(1:end-1) );

%Generate figure
fig = figure('name','rectangle1','color',[1 1 1]);

%Patch green rectangles to represent the rectangular strips
for n=1:N
    xx = [x(n), x(n+1), x(n+1), x(n) ];
    yy = [0, 0, y(n), y(n)];
    patch( xx, yy, 'g' );
    hold on;
end

%Overlay x,y data
plot(x,y,'r+');

%Set labels and title
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
title(['Rectangle1 integration of x,y data. I = ',num2str(I) ],'fontsize',fsize );
set(gca,'fontsize',fsize);
grid on

%%

%xygen
% Function which generates some x,y data for use in numeric integration
% functions. N points are returned on interval [a,b].
function [x,y,I] = xygen(a,b,N)

%Generate a vector of x data
x = linspace(a,b,N);

%Generate a vector of y data
y = sin( 2*2*pi*x/10 ) - 3*exp(-x) + 3;

%Evaluate definite integral between limits a and b
I = -(10/(2*2*pi))*cos( 2*2*pi*b/10) + 3*exp(-b) + 3*b - ...
    ( -(10/(2*2*pi))*cos( 2*2*pi*a/10) + 3*exp(-a) + 3*a );

%End of code