%Test of Simpson's (Quadratic curve section) numeric integration method.
% LAST UPDATED by Andy French Dec 2021.
function simpson_test

%Number of strips - THIS NEEDS TO BE AN EVEN NUMBER
N = 6;

%Generate test x,y data
[x,y,Iexact] = xygen(3,7,N+1);
[xx,yy] = xygen(0,10,1000);

%Perform numeric integration
[I,fig] = simpson(x,y);

%Overlay original curve
plot(xx,yy);

%Save graph as .png file
print( gcf, ['simpson ',num2str(N),'.png'],'-dpng','-r300'); close(gcf);

%Percentage error of integral
disp(' ')
disp(' SIMPSON RULE ')
disp([' Fractional error of integral = ',num2str( 100*(I-Iexact)/Iexact ),'%'] );
disp(' ')

%%

%simpson
% Numeric integration via Simpson's rule. Note the number of strips must be
% even i.e. the number of data points must be odd.
%
% Syntax: [I,fig] = simpson(x,y)
%
% x,y vectors of plot data. Assume that the x values are increasing
% and they are equally spaced. The number of y elements must equal the
% number of x elements.
%
% I is the estimate of the integral between x(1) and x(end)
% fig is a handle of the plot figure which is generated.

function [I,fig] = simpson(x,y)

%Fontsize
fsize = 18;

%Define number of data points for quadratic curve between strip ends
M = 100;

%Determine strip width h. (Assume constant!)
N = numel(x) - 1;
h = ( x(end) - x(1) )/N;

%Compute estimate of integral
odd = 3:2:N-1;
even = 2:2:N;
I = (1/3)*h*( y(1) + 4*sum( y(even) ) + 2*sum( y(odd) ) + y(end) );

%Generate figure
fig = figure('name','simpson','color',[1 1 1]);

%Patch green polygons strips to represent the quadratic segments
n=1;
for p=1:N/2
    xxx = linspace( x(n), x(n+2), M );
    xx = [ xxx, x(n+2), x(n) ];
    yy = [ simpquad( xxx, x(n), x(n+1), x(n+2), y(n), y(n+1), y(n+2) ), 0, 0];
    patch( xx, yy, 'g' );
    hold on;
    plot( [x(n+1), x(n+1)], [0, y(n+1)], 'k-' );
    n = n + 2;
end

%Overlay x,y data
plot(x,y,'r+');

%Set labels and title
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
title([num2str(N),' strip Simpson rule integration of x,y data. I = ',num2str(I) ],'fontsize',fsize );
set(gca,'fontsize',fsize);
grid on;

%%

%simpquad
% Function which evaluates quadratic segment associated with Simpson's rule
function y = simpquad( x, x0,x1,x2, y0, y1,y2 )
y = y0*(x - x1).*(x-x2)/( (x0-x1)*(x0-x2) ) + ...
    y1*(x - x0).*(x-x2)/( (x1-x0)*(x1-x2) ) + ...
    y2*(x - x0).*(x-x1)/( (x2-x0)*(x2-x1) ) ;
    
%%

%xygen
% Function which generates some x,y data for use in numeric integration
% functions. N points are returned on interval [a,b].
function [x,y,I] = xygen(a,b,N)

%Generate a vector of x data
x = linspace(a,b,N);

%Generate a vector of y data
y = sin( 2*2*pi*x/10 ) - 3*exp(-x) + 3;

%Evaluate definite integral between limits a and b
I = -(10/(2*2*pi))*cos( 2*2*pi*b/10) + 3*exp(-b) + 3*b - ...
    ( -(10/(2*2*pi))*cos( 2*2*pi*a/10) + 3*exp(-a) + 3*a );

%End of code