%Planck law integral calculator and curve plotter
% Andy French 16/12/2021

function planck_calculator

%Temperature /K
T = 5778;

%Scaling factor (e.g. radiative power received at the Earth due to the Sun)
Rsun = 696340e3;
R = 149597870700;
SF = (Rsun/R)^2;

%Overlay solar spectra data
overlay_solar_spectra = 1;

%Range of optical wavelengths /nm
lambda1 = 380; lambda2 = 740;

%Max wavelength /nm for curve plot
max_lambda = 1500;

%Boltzmann's constant /m^2kgs^-2K^-1
kB = 1.3806504e-23;

%Planck's constant /m^2kgs^-1
h = 6.626068e-34;

%Speed of light /ms^-1
c = 299792458;

%Fontsize for graphs
fsize = 18;

%Number of lambda evaluation points
N = 1000;

%%

%%% Planck spectrum calculator %%%

%Determine Planck curve integral limits
xplus = h*c/(kB*T*lambda1*1e-9);
xminus = h*c/(kB*T*lambda2*1e-9);

%Characteristic wavelength /nm
lambda_characteristic = (h*c/(kB*T))/1e-9;

%Determine Planck curve integrals via a cubic spline method
x = linspace(0,2*xplus,1000);
I2 = integrate( xminus,xplus, x, (x.^2)./( exp(x) - 1 ) );
I3 = integrate( xminus,xplus, x, (x.^3)./( exp(x) - 1 ) );

%Determine average wavelength /nm over range of wavelengths
mean_lambda = lambda_characteristic*I2/I3;

%Compute Stefan-Boltzmann constant
sigma = 2*(pi^5)*(kB^4)/(15*(h^3)*(c^2));

%Compute total radiation flux /Wm^-2
I = sigma*T^4;

%Display Planck calculator output
disp(' ')
disp('%%%%% PLANCK SPECTRUM CALCULATOR %%%%%')
disp(' ')
disp([ 'T = ',num2str(T),'K, lambda1 = ',num2str(lambda1),'nm, lambda2 = ',num2str(lambda2),'nm.' ] )
disp([ 'I = ',num2str(I,4),'Wm^-2, mean lambda = ',num2str(mean_lambda,4),'nm.' ] );
disp(' ');

%

%%% Plot Planck curve %%%

%Compute irradiance in Wm^-2 /m
lambda = linspace(0,max_lambda,N);

%Compute wavelength /nm corresponding to peak of Planck law
lambda_max = (1e9)*h*c./(4.9651*kB*T);

%Plot Planck law, with visible colours shown.
hold on;
for n=2:N
    %Plot coloured patch to denote visible colours
    f_THz = ( c/(lambda(n)*1e-9) )/1e12;
    [R,G,B,colour_str] = colours_from_f(f_THz);
    if isnan(R)==0
        patch( [lambda(n-1),lambda(n),lambda(n),lambda(n-1)],...
            SF*[0,0,planck(lambda(n),T),planck(lambda(n-1),T) ],...
            [R,G,B],'edgecolor', [R,G,B] );
    end
end
plot( lambda, SF*planck(lambda,T),'k-','linewidth',2);
grid on; box on;
xlabel('Wavelength (nm)','fontsize',fsize );
ylabel('Irradiance ( Wm^{-2}/nm )','fontsize',fsize );
title(['T=',num2str(T),'K, \lambda_{max}=',...
    num2str(lambda_max,4),'nm, \sigmaT^4=',num2str(I,4),'Wm^{-2}'],'fontsize',fsize );
set(gca,'fontsize',fsize );

%Overlay solar spectra data
if overlay_solar_spectra ==1
    [num,txt,raw] = xlsread('solar_spectra.xlsx');
    lambda_sol_nm = num(:,1)*1000;
    spectral_intensity = num(:,2)/1000;
    plot( lambda_sol_nm, spectral_intensity, 'b+' );
end
xlim([0,max_lambda]);
print( gcf, 'planck spectrum.png','-dpng','-r300' );
close(gcf);

%

%Planck irradiance (Wm^-2 /nm) given T in K and lambda in nm
function B = planck(lambda,T)

h = 6.626068e-34; %Planck's constant /m^2kgs^-1
kB = 1.3806504e-23; %Boltzmann's constant /m^2kgs^-2K^-1
c = 299792458; %Speed of light /ms^-1

L = lambda*1e-9;
B = (2*pi*h*c^2)./(L.^5);
B = B./( exp( h*c./( L.*T*kB ) ) - 1 );

%Convert this to Wm^-2 /nm
B = (1e-9)*B;

%%

%colours_from_f
% Function which provides the R,G,B values( within interval [0,1] )
% of visible light depending on the frequency /THz
function [R,G,B,colour_str] = colours_from_f(f)
if f < 405
    R = NaN; G = NaN; B = NaN; colour_str = 'Infra Red';
elseif (f>=405) && ( f < 480 )
    R = 1; G = 0; B = 0; colour_str = 'Red';
elseif (f>=480) && ( f < 510 )
    R = 1; G = 127/255; B = 0; colour_str = 'Orange';
elseif (f>=510) && ( f < 530 )
    R = 1; G = 1; B = 0; colour_str = 'Yellow';
elseif (f>=530) && ( f < 600 )
    R = 0; G = 1; B = 0; colour_str = 'Green';
elseif (f>=600) && ( f < 620 )
    R = 0; G = 1; B = 1; colour_str = 'Cyan';
elseif (f>=620) && ( f < 680 )
    R = 0; G = 0; B = 1; colour_str = 'Blue';
elseif (f>=680) && ( f <= 790 )
    R = 127/255; G = 0; B = 1; colour_str = 'Violet';
else
    R = NaN; G = NaN; B = NaN; colour_str = 'Ultra Violet';
end

%%

%integrate
% Function that fits a piecwise polynomial to an array of (x,y) data and
% thus computes the definite integral between the points x=a and x=b.
% Note a and b can be arrays, but must be of the same dimensions.
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.
%
% NOTE: seriously erroneous results may occur if a,b values are outside
% the range of array x.
%
% LAST UPDATED by Andrew French. 27/1/2016.
%
% Syntax: I = integrate(a,b,x,y)
%
% x   - Vector of data x variable
% y   - Vector of data y(x) variable
% a   - Array of x starting values for the definite integral of y vs x
% b   - Array of x finishing values for the definite integral of y vs x

function I = integrate(a,b,x,y)
warning off
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define type of piecewise polynomial
% computed. Hermite polynomials tend to be better for
% non-smooth data and tends to be 'shape preserving.'

%poly = 'spline';
poly = 'hermite';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Compute piecewise polynomial. Cubic spline is the default
if strcmp(poly,'hermite')==1
    pp=pchip(x,y);
else
    pp=spline(x,y);
end

%Take apart piecewise polynomial
[br,co,npy,nco]=unmkpp(pp);

%Scale factors for integration
sf=nco:-1:1;

%Integral coefficients
ico=[co./sf(ones(npy,1),:) zeros(npy,1)];

%Integral spline has higher order
nco=nco+1;

%Find constant terms in polynomials
for k=2:npy
    ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
end
ico(1,nco)=0;

%Build pp form for integral
ppi=mkpp(br,ico);

%Evaluate integral at values of a and b
Ia = ppval(ppi,a);
Ib = ppval(ppi,b);

%Define output
I = Ib - Ia;

%End of code